Imports System.Windows.Forms
Imports System.Drawing.Imaging
Imports Microsoft.Win32

Public Class OptionsDialog

#Region " Methods "

    Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me.FormatComboBox.Items.Add(ImageFormat.Png)
        Me.FormatComboBox.Items.Add(ImageFormat.Jpeg)
        Me.FormatComboBox.Items.Add(ImageFormat.Bmp)
        Me.FormatComboBox.Items.Add(ImageFormat.Gif)
        Me.FormatComboBox.Items.Add(ImageFormat.Tiff)
    End Sub

    Public Sub SetValues()
        Me.FormatComboBox.SelectedItem = My.Settings.AutoFileFormat
        Me.AutoFileNameTextBox.Text = My.Settings.AutoGeneratedName
        Me.InstantMenuCheckBox.Checked = My.Settings.InstantMenu
        Me.SoundCheckBox.Checked = My.Settings.PlaySound
        If My.Settings.FileSaving = SaveFile.Manual Then
            Me.SaveFileManuallyRadioButton.Checked = True
        Else
            Me.SaveFileAutomaticallyRadioButton.Checked = True
        End If
        If My.Settings.SendTo = SendTo.File Then
            Me.SendToFileRadioButton.Checked = True
        Else
            Me.SendToClipboardRadioButton.Checked = True
        End If
        Me.UpdateCheckBox.Checked = My.Settings.CheckForUpdate
        Dim keyName As String = My.Computer.Registry.CurrentUser.Name & "\Software\Microsoft\Windows\CurrentVersion\Run"
        If My.Computer.Registry.GetValue(keyName, "ScreenShot", Nothing) IsNot Nothing Then
            Me.StartUpCheckBox.Checked = True
        Else
            Me.StartUpCheckBox.Checked = False
        End If
        Me.IncludeCursorCheckBox.Checked = My.Settings.IncludeCursor
        Me.FolderBrowserDialog1.SelectedPath = My.Settings.DirectoryPath
        Me.JpegQualityTrackBar.Value = My.Settings.Quality
        Me.QualityLabel.Text = CStr(Me.JpegQualityTrackBar.Value) & "%"
        Me.FullScreenModeHKLabel.Hotkey = New Hotkey(100, My.Settings.FullScreenModeHK)
        Me.ObjectStartModeHKLabel.Hotkey = New Hotkey(101, My.Settings.ObjectStartModeHK)
        Me.ObjectEndModeHKLabel.Hotkey = New Hotkey(102, My.Settings.ObjectEndModeHK)
        Me.RectangleModeHKLabel.Hotkey = New Hotkey(103, My.Settings.RectangleModeHK)
        Me.ModeCaptureHKLabel.Hotkey = New Hotkey(104, My.Settings.ModeCaptureHK)
        Me.ActiveWindowCaptureHKLabel.Hotkey = New Hotkey(105, My.Settings.ActiveWindowHK)
    End Sub

    Private Sub SetEnable()
        Me.SaveGroupBox.Enabled = Me.SendToFileRadioButton.Checked
        Me.AutoGeneratedGroupBox.Enabled = (Me.SendToFileRadioButton.Checked AndAlso Me.SaveFileAutomaticallyRadioButton.Checked)
        If Me.SaveFileManuallyRadioButton.Checked Then
            Me.JpegQualityGroupBox.Enabled = Me.SendToFileRadioButton.Checked
        Else
            Me.JpegQualityGroupBox.Enabled = Me.SendToFileRadioButton.Checked _
            AndAlso (CType(Me.FormatComboBox.SelectedItem, ImageFormat).Equals(Imaging.ImageFormat.Jpeg))
        End If
    End Sub

    Private Shared Function RegisterHotKey(ByVal hk As Hotkey) As Hotkey
        With My.Forms.MenuForm.HotkeyManager
            If hk.Key <> Keys.None Then
                Try
                    If .ContainsId(hk.Id) Then
                        .Replace(hk, True)
                    Else
                        .RegisterHotKey(hk, True)
                    End If
                Catch ex As Exception
                    MessageBox.Show(hk.ToString & " Screen Shot hotkey failed to register.", _
                    "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                    MenuForm.Log.WriteException(ex, TraceEventType.Warning)
                End Try
            End If
            Return hk
        End With
    End Function

    Private Sub Apply()
        My.Settings.AutoGeneratedName = Me.AutoFileNameTextBox.Text
        My.Settings.AutoFileFormat = CType(Me.FormatComboBox.SelectedItem, ImageFormat)
        My.Settings.InstantMenu = Me.InstantMenuCheckBox.Checked
        My.Settings.PlaySound = Me.SoundCheckBox.Checked
        My.Settings.IncludeCursor = Me.IncludeCursorCheckBox.Checked
        If Me.SaveFileManuallyRadioButton.Checked Then
            My.Settings.FileSaving = SaveFile.Manual
        Else
            My.Settings.FileSaving = SaveFile.Automatic
        End If
        If Me.SendToFileRadioButton.Checked Then
            My.Settings.SendTo = SendTo.File
        Else
            My.Settings.SendTo = SendTo.Clipboard
        End If
        My.Settings.CheckForUpdate = Me.UpdateCheckBox.Checked
        Dim rk As Microsoft.Win32.RegistryKey
        rk = My.Computer.Registry.CurrentUser.OpenSubKey("Software\Microsoft\Windows\CurrentVersion\Run", True)
        If Me.StartUpCheckBox.Checked = True Then
            rk.SetValue("ScreenShot", Application.ExecutablePath)
        Else
            rk.DeleteValue("ScreenShot", False)
        End If
        My.Settings.Quality = Me.JpegQualityTrackBar.Value
        My.Settings.DirectoryPath = Me.FolderBrowserDialog1.SelectedPath
        If My.Settings.FullScreenModeHK <> Me.FullScreenModeHKLabel.Hotkey.Data Then
            My.Settings.FullScreenModeHK = OptionsDialog.RegisterHotKey(Me.FullScreenModeHKLabel.Hotkey).Data
        End If
        If My.Settings.ObjectStartModeHK <> Me.ObjectStartModeHKLabel.Hotkey.Data Then
            My.Settings.ObjectStartModeHK = OptionsDialog.RegisterHotKey(Me.ObjectStartModeHKLabel.Hotkey).Data
        End If
        If My.Settings.ObjectEndModeHK <> Me.ObjectEndModeHKLabel.Hotkey.Data Then
            My.Settings.ObjectEndModeHK = OptionsDialog.RegisterHotKey(Me.ObjectEndModeHKLabel.Hotkey).Data
        End If
        If My.Settings.RectangleModeHK <> Me.RectangleModeHKLabel.Hotkey.Data Then
            My.Settings.RectangleModeHK = OptionsDialog.RegisterHotKey(Me.RectangleModeHKLabel.Hotkey).Data
        End If
        If My.Settings.ModeCaptureHK <> Me.ModeCaptureHKLabel.Hotkey.Data Then
            My.Settings.ModeCaptureHK = OptionsDialog.RegisterHotKey(Me.ModeCaptureHKLabel.Hotkey).Data
        End If
        If My.Settings.ActiveWindowHK <> Me.ActiveWindowCaptureHKLabel.Hotkey.Data Then
            My.Settings.ActiveWindowHK = OptionsDialog.RegisterHotKey(Me.ActiveWindowCaptureHKLabel.Hotkey).Data
        End If
        My.Settings.Save()
        Me.ApplyButton.Enabled = False
    End Sub

    Private Function IsModified() As Boolean
        If My.Settings.DirectoryPath <> Me.FolderBrowserDialog1.SelectedPath Then
            Return True
        End If
        If My.Settings.InstantMenu <> Me.InstantMenuCheckBox.Checked Then
            Return True
        End If
        If My.Settings.CheckForUpdate <> Me.UpdateCheckBox.Checked Then
            Return True
        End If
        Dim rk As RegistryKey = My.Computer.Registry.CurrentUser.OpenSubKey("Software\Microsoft\Windows\CurrentVersion\Run", False)
        If (Array.IndexOf(rk.GetValueNames, "ScreenShot") <> -1) <> Me.StartUpCheckBox.Checked Then
            Return True
        End If
        Dim sendToValue As SendTo
        If Me.SendToFileRadioButton.Checked Then
            sendToValue = SendTo.File
        Else
            sendToValue = SendTo.Clipboard
        End If
        If My.Settings.SendTo <> sendToValue Then
            Return True
        End If
        Dim saveFileValue As SaveFile
        If Me.SaveFileManuallyRadioButton.Checked Then
            saveFileValue = SaveFile.Manual
        Else
            saveFileValue = SaveFile.Automatic
        End If
        If My.Settings.FileSaving <> saveFileValue Then
            Return True
        End If
        If My.Settings.PlaySound <> Me.SoundCheckBox.Checked Then
            Return True
        End If
        If My.Settings.IncludeCursor <> Me.IncludeCursorCheckBox.Checked Then
            Return True
        End If
        If My.Settings.AutoGeneratedName <> Me.AutoFileNameTextBox.Text Then
            Return True
        End If
        If Not My.Settings.AutoFileFormat.Equals(CType(Me.FormatComboBox.SelectedItem, ImageFormat)) Then
            Return True
        End If
        If My.Settings.Quality <> Me.JpegQualityTrackBar.Value Then
            Return True
        End If
        If My.Settings.FullScreenModeHK <> Me.FullScreenModeHKLabel.Hotkey.Data Then
            Return True
        End If
        If My.Settings.ObjectStartModeHK <> Me.ObjectStartModeHKLabel.Hotkey.Data Then
            Return True
        End If
        If My.Settings.ObjectEndModeHK <> Me.ObjectEndModeHKLabel.Hotkey.Data Then
            Return True
        End If
        If My.Settings.RectangleModeHK <> Me.RectangleModeHKLabel.Hotkey.Data Then
            Return True
        End If
        If My.Settings.ModeCaptureHK <> Me.ModeCaptureHKLabel.Hotkey.Data Then
            Return True
        End If
        If My.Settings.ActiveWindowHK <> Me.ActiveWindowCaptureHKLabel.Hotkey.Data Then
            Return True
        End If
        Return False
    End Function

#End Region

#Region " Events "

    Private Sub DirectoryButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DirectoryButton.Click
        Me.FolderBrowserDialog1.SelectedPath = My.Settings.DirectoryPath
        Me.FolderBrowserDialog1.ShowDialog()
        Me.ApplyButton.Enabled = Me.IsModified
    End Sub

    Private Sub TrackBar1_ValueChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles JpegQualityTrackBar.ValueChanged
        Me.QualityLabel.Text = CStr(Me.JpegQualityTrackBar.Value) & "%"
    End Sub

    Private Sub Ok_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ok_Button.Click
        Me.Apply()
        Me.DialogResult = Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub

    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Private Sub FullScreenHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FullScreenHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Full Screen Mode"
        hotKeyFrm.Hotkey = Me.FullScreenModeHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.FullScreenModeHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub ObjectStartHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ObjectStartHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Object Start Mode"
        hotKeyFrm.Hotkey = Me.ObjectStartModeHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.ObjectStartModeHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub ObjectEndHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ObjectEndHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Object End Mode"
        hotKeyFrm.Hotkey = Me.ObjectEndModeHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.ObjectEndModeHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub RectangleHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RectangleHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Rectangle Mode"
        hotKeyFrm.Hotkey = Me.RectangleModeHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.RectangleModeHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub ModeCaptureHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModeCaptureHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Current Mode Capture"
        hotKeyFrm.Hotkey = Me.ModeCaptureHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.ModeCaptureHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub ActiveWindowCaptureHKButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActiveWindowCaptureHKButton.Click
        Dim hotKeyFrm As New HotKeyDialog
        hotKeyFrm.Text = "Hotkey - Active Window Capture"
        hotKeyFrm.Hotkey = Me.ActiveWindowCaptureHKLabel.Hotkey
        If hotKeyFrm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            Me.ActiveWindowCaptureHKLabel.Hotkey = hotKeyFrm.Hotkey
        End If
    End Sub

    Private Sub PropertiesDialog_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.SetValues()
        Me.Options_TabControl.SelectedIndex = 0
        Me.SetEnable()
        Me.ApplyButton.Enabled = Me.IsModified
    End Sub

    Private Sub GeneratedNameTextBox_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles AutoFileNameTextBox.KeyUp
        If e.KeyCode = Keys.Enter Then
            Me.JpegQualityTrackBar.Focus()
        End If
    End Sub

    Private Sub AutoFileNameRadioButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveFileAutomaticallyRadioButton.CheckedChanged, SaveFileManuallyRadioButton.CheckedChanged
        Me.AutoGeneratedGroupBox.Enabled = Me.SaveFileAutomaticallyRadioButton.Checked
        Me.JpegQualityGroupBox.Enabled = SaveFileManuallyRadioButton.Checked OrElse ( _
        Me.SaveFileAutomaticallyRadioButton.Checked AndAlso CType(Me.FormatComboBox.SelectedItem, ImageFormat).Equals(ImageFormat.Jpeg))
        Me.ApplyButton.Enabled = Me.IsModified
    End Sub

    Private Sub FormatComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FormatComboBox.SelectedIndexChanged
        Me.JpegQualityGroupBox.Enabled = CType(Me.FormatComboBox.SelectedItem, ImageFormat).Equals(ImageFormat.Jpeg)
    End Sub

    Private Sub ClipboardRadioButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SendToClipboardRadioButton.CheckedChanged, SendToFileRadioButton.CheckedChanged
        Me.SetEnable()
        Me.ApplyButton.Enabled = Me.IsModified
    End Sub

    Private Sub OptionsDialog_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        My.Forms.MenuForm.MouseHook.RemoveHook()
        My.Forms.MenuForm.MenuMotionOn(True, True)
        If My.Forms.RectangleForm.Visible Then
            My.Forms.RectangleForm.Close()
        End If
    End Sub

    Private Sub ApplyButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ApplyButton.Click
        Me.Apply()
    End Sub

    Private Sub Modified_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartUpCheckBox.CheckedChanged, _
    UpdateCheckBox.CheckedChanged, InstantMenuCheckBox.CheckedChanged, RectangleModeHKLabel.HotkeyChanged, _
    ObjectStartModeHKLabel.HotkeyChanged, ObjectEndModeHKLabel.HotkeyChanged, ModeCaptureHKLabel.HotkeyChanged, FullScreenModeHKLabel.HotkeyChanged, _
    ActiveWindowCaptureHKLabel.HotkeyChanged, AutoFileNameTextBox.TextChanged, JpegQualityTrackBar.Scroll, FormatComboBox.TextChanged, SoundCheckBox.CheckedChanged, IncludeCursorCheckBox.CheckedChanged
        Me.ApplyButton.Enabled = Me.IsModified
    End Sub

    Private Sub AutoFileNameTextBox_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles AutoFileNameTextBox.Validating
        If Not Me.AutoFileNameTextBox.Text = String.Empty Then
            For Each ch As Char In IO.Path.GetInvalidFileNameChars
                If Me.AutoFileNameTextBox.Text.Contains(ch) Then
                    e.Cancel = True
                    MessageBox.Show("The '" & ch & "' is invalid file-name character!" _
                    & Environment.NewLine & "Please enter valid file-name.", "Invalid File Name", _
                    MessageBoxButtons.OK, MessageBoxIcon.Warning)
                End If
            Next
        Else
            e.Cancel = True
            MessageBox.Show("The file-name should contain at least one character!" _
            & Environment.NewLine & "Please enter valid file-name.", "Invalid File Name", _
            MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End If
    End Sub

#End Region

End Class
